#!/usr/bin/env python
import argparse
from smurfs._smurfs.smurfs import Smurfs
from smurfs._smurfs.multi_smurfs import MultiSmurfs
from smurfs.__version__ import __version__
from smurfs.support.mprint import *
import os
import pickle
import subprocess
import sys
import warnings


def get_parser():
    parser = argparse.ArgumentParser()

    parser.add_argument("target", help="This parameter represents the target analyzed by SMURFS. This parameter is "
                                       "rather vague, to give you the maximum amount of flexibility. Either you can "
                                       "provide any name of a star (resolvable through Simbad) that has been observed "
                                       "by TESS/Kepler/K2. You can also provide a filename through this parameter.\n\n"
                                       "If you choose to do the the first, the code prioritizes TESS over all "
                                       "other missions (if you don't provide the mission parameter), and SC over LC "
                                       "observations. It then checks MAST if there are SC observations "
                                       "from TESS for this star. If there are, it uses "
                                       "*lightkurve.search_lightcurvefile* to download those."
                                       " If there are none, it checks if there are "
                                       "LC observations from TESS and uses Eleanor to extract the light curves "
                                       "from the FFIs. If there are None, it searches all other missions "
                                       "for light kurves of this object.If you choose another "
                                       " mission, it will call *lightkurve.search_lightcurvefile* for this specific "
                                       "mission.\n\nIf you choose to provide "
                                       "a file through this parameter, make sure that you follow these conventions:\n\n"
                                       "- The file must be an ASCII file\n\n"
                                       "- The first column must contain the time stamps\n\n"
                                       "- The second column must contain the flux\n\n"
                                       "- If a third column exists, SMURFS will assume that these are the uncertainties in the flux."
                                       "\n\n\nSMURFS will take the file as is and won't apply "
                                       "any corrections to it, if you don't use the *apply_corrections* flag. "
                                       "It then assumes that the flux values are in magnitude and varying "
                                       "around 0, that the time stamps are in days and that the "
                                       "data set is properly reduced.", type=str)
    parser.add_argument("snr",
                        help="This parameter represents the lower bound signal to noise ratio any frequency must have. SMURFS "
                             "computes the SNR by taking the amplitude of an individual frequency (as defined by the amplitude in the frequency "
                             "spectrum). Next, it applies half of the window size to either side of the frequency, starting by the next adjacent "
                             "minima, and takes the mean of this window as the noise surrounding the frequency. The ratio of these two values is "
                             "the resulting SNR for an individual frequency. By default, SMURFS stops its run when the first frequency less than "
                             "this value has been found",
                        type=float)
    parser.add_argument("windowSize", help="The window size used to get the SNR for a given frequency. SMURFS "
                                           "defines the window as half of the given value on each side of the peak "
                                           "in the periodogram, starting from the first minima next to the peak.",
                        type=float)
    parser.add_argument("-fr", "--frequencyRange",
                        help="Setting this parameter, allows you to restrict the analysis "
                             "of a time series to a given range in the frequency spectrum. This might be useful, if yo have a lot of high "
                             "amplitude noise in another part of the spectrum, than the one you might be interested in. Provide the parameter "
                             "in the form *0,100*.",
                        type=str, default="None,None")

    parser.add_argument("-ssa", "--skipSimilarFrequencies",
                        help="Ignores regions where SMURFS finds multiple frequencies within a small range. "
                             "This can happen due to insufficient fits or signals that are hard to fit.",
                        action='store_true')

    parser.add_argument("-sc", "--skipCutoff",
                        help="By default, SMURFS stops the extraction of frequencies if it finds 10 frequencies in a row"
                             "that have a standard deviation of <0.05, as it assumes it can't work itself out of "
                             "this region. You can override this behaviour by setting this flag. Be aware, that "
                             "this might lead to unknown behaviour. Usually, instead of setting this flag, it is "
                             "better to set the -ssa flag",
                        action='store_true')

    parser.add_argument("-ef", "--extendFrequencies",
                        help="Extends the analysis by n frequencies. By default, SMURFS stops when it finds the first "
                             "insignificant frequency. Setting this parameter requires SMURFS to find n insignificant "
                             "frequencies in a row.", type=int, default=0)
    parser.add_argument("-fd","--frequencyDetection",help="If this value is set, a found frequency is compared to the "
                                                          "original periodogramm. If the ratio between the "
                                                          "amplitude of the found frequency and the maximum in "
                                                          "the range of the found frequency on the original "
                                                          "periodogram is lower than the set value, it will "
                                                          "ignore this frequency range going forward.",type=float,
                        default=None)

    parser.add_argument("-imf", "--improveFitMode",
                        help="This parameter defines the way SMURFS uses a Period04 like improvement of frequencies."
                             " You can set the following modes:\n\n"
                             "- *all*: Tries to refit all found frequencies after every frequeny that is found. This is the usual behaviour of Period04 and the default setting.\n\n"
                             "- *end*: Improves the frequencies after SMURFS would stop its run\n\n"
                             "- *none*: Disables the improve frequencies setting. This can be useful if you find a lot of frequencies and the run would take an unnecessary amount of time\n\n"
                        , type=str,
                        choices=['all', 'end', 'none'],
                        default='all')

    parser.add_argument("-fm", "--fitMethod",
                        help="SMURFS implements two different fitting libraries: Either it uses *scipy* "
                             "(scipy.optimize.curve_fit) or *lmfit* (lmfit Model fit). Choosing one over the other "
                             "might lead to different results, so if you have find an unexpected result, try "
                             "to switch the fitting mode.",
                        type=str, choices=["scipy", "lmfit"], default="lmfit")

    parser.add_argument("-ft", "--fluxType",
                        help='The TESS mission gives its end users different data products to choose, if you download '
                             'them directly from MAST. You can pass the type of data product you like using this '
                             'parameter. For the SC data, where the light curve is preprocessed by SPOC, you can choose'
                             'two different products: \n\n'
                             '- *SAP flux*: SAP is the simple aperture photometry flux (resulting light curve is the flux after summing the calibrated pixels within the TESS optimal aperture)\n\n'
                             '- *PDCSAP flux*: PDCSAP is the Pre-search Data conditioned Simple aperture photometry, which is corrected using co-trending basis vectors.'
                             '\n\n\nBy default we use the PDCSAP flux, but you can also '
                             'choose another one if you like.\n\n\nIf your target is only observed in LC mode, SMURFS '
                             'also provides these two modes (these have a slightly different meaning, their '
                             'result is however equivalent to the SC SAP and PDSCAP fluxes, see the Eleanor '
                             'documentation). However, in LC mode you have also the PSF flux type, which models a '
                             'point spread function for a given star. The validation page generated when SMURFS '
                             'extracts a target from a FFI always shows the SAP and the chosen flux to compare. You '
                             'might want to try different settings, depending on your use case',
                        type=str, choices=["PDCSAP", "SAP", "PSF"], default="PDCSAP")
    parser.add_argument("-pca", "--do_pca",
                        help="Activates the PCA analysis (aperture × TPF + background subtraction + "
                             "cotrending basis vectors). This doesn't change the data you are trying to analyze, "
                             "but shows the PDSCAP flux in the validation page. Only applicable "
                             "when using LC data.", action='store_true')
    parser.add_argument("-psf", "--do_psf", help="Activates the PSF analysis. This adds point spread function "
                                                 "modelling to the extraction of light curves from FFIs.  This doesn't "
                                                 "change the data you are trying to analyze, "
                                                 "but shows the PSF flux in the validation page. Only applicable "
                                                 "when using LC data.", action='store_true')

    parser.add_argument("-so", "--storeObject", help="If this flag is set, the SMURFS object is stored in the "
                                                     "results. You can later use this object to load the "
                                                     "_result into a python file using 'Smurfs.from_path'. Allows "
                                                     "you to easily access all convenience functions from SMURFS."
                        , action='store_true')

    parser.add_argument("-sp", "--savePath", help="Allows you to set the save path of the analysis. By "
                                                  "default it will save it in the same folder, where "
                                                  "the module was called.", type=str, default=".")

    parser.add_argument("-i", "--interactive", help="Using this flag will automatically start an iPython shell. "
                                                    "This allows for direct interaction with the result using the "
                                                    "'star' object. You can then access all convenience functions "
                                                    "directly (like plotting, the FrequencyFinder object, etc)."
                        , action='store_true')

    parser.add_argument("-m", "--mission", help="Three different missions are available: Kepler,TESS,K2. You can choose"
                                                " the mission by setting this value. By default, only TESS missions are"
                                                " considered"
                        , type=str, choices=["Kepler", "TESS", "K2"], default="TESS")

    parser.add_argument("-cl", "--sigmaClip", help="Sets the sigma for the sigma clipping. Default is 4.", type=float,
                        default=4)
    parser.add_argument("-it", "--iters", help="Sets the iterations for the sigma clipping. Default is 1.", type=int,
                        default=1)

    parser.add_argument("-ac", "--apply_corrections",
                        help="If this flag is set, correction (sigma clipping, conversion"
                             " to magnitude) are applied to files. Make sure that your flux is in "
                             "electron counts if you use this flag.", action='store_true')

    """
    #todo replace this
    parser.add_argument("-om", "--outputMode", help="Optional parameter that defines what will be exported as dataset. "
                                                    "There are two possibilities: Normal and Full. Normal mode means, that"
                                                    "only the first and last amplitudespectra will be plotted & exported, "
                                                    "full means all amplitudespectra will be plotted & exported. Default is "
                                                    "Normal.",
    
                        type=str, choices=["Normal", "Full"], default="Normal")
                        
    parser.add_argument("-igr", "--ignoreCutoffRatio", help="Optional parameter. If this is set to True, it will ignore"
                                                            "the gap ratio cutoff criterion", action='store_true')  
                                                            
    parser.add_argument("-trs", "--timeBaseSplit",
                        help="Optional parameter that describes the split within the time range."
                             "This can be used to get multiple amplitude spectra for the "
                             "lightcurve, split up within the time Range. A parameter could be "
                             "for example 50, where the parameter would split the lightcurve in"
                             "50 day chunks (last segment will be smaller/equal). Defaults"
                             "to -1, which means no splitting will occur",
                        type=int, default=-1)
    parser.add_argument("-o", "--overlap",
                        help="Optional parameter that describes the overlap of two split ranges. Will only"
                             "work, if -trs is set to a reasonable parameter. For example, if the trs is "
                             "set to 50 and the overlap to 2, the chunks would contain the ranges (0,50),"
                             "(48,98),(96,146) ... Defaults to 0, which means no overlap will take place",
                        type=int, default=0)                                                                                  
    """
    parser.add_argument("--version", help="Shows version of SMURFS", action='version',
                        version='SMURFS {version}'.format(version=__version__))
    return parser


def main(args=None):
    if args is None:
        args = sys.argv[1:]

    parser = get_parser()
    args = parser.parse_args(args)

    if "," in args.target:
        targets = args.target.split(",")
    else:
        targets = [args.target]

    f_min = None if args.frequencyRange.split(",")[0] == 'None' else float(args.frequencyRange.split(",")[0])
    f_max = None if args.frequencyRange.split(",")[1] == 'None' else float(args.frequencyRange.split(",")[1])

    with warnings.catch_warnings():
        warnings.simplefilter("ignore")
        if len(targets) == 1:
            target = targets[0]

            if os.path.exists(target) and os.path.isfile(target):
                s = Smurfs(file=target, flux_type=args.fluxType, mission=args.mission, sigma_clip=args.sigmaClip,
                           iters=args.iters, do_pca=args.do_pca, do_psf=args.do_psf,
                           apply_file_correction=args.apply_corrections)
            else:
                s = Smurfs(target_name=target, flux_type=args.fluxType, mission=args.mission, sigma_clip=args.sigmaClip,
                           iters=args.iters, do_pca=args.do_pca, do_psf=args.do_psf)

            improve_fit = args.improveFitMode == 'all'

            s.run(snr=args.snr, window_size=args.windowSize, f_min=f_min, f_max=f_max,
                  skip_similar=args.skipSimilarFrequencies, similar_chancel=not args.skipCutoff
                  , extend_frequencies=args.extendFrequencies, improve_fit=improve_fit
                  , mode=args.fitMethod,frequency_detection=args.frequencyDetection)

            if args.improveFitMode == 'end':
                mprint("Improving fit ...", state)
                s.improve_result()

            s.save(args.savePath, args.storeObject)

            # start interactive shell, by loading smurfs object directly into the shell
            if args.interactive:
                mprint("Starting interactive shell. Use the 'star' object to interact with the result!", state)
                pickle.dump(s, open("i_obj.smurfs", "wb"))
                cmd = ["ipython", "-i", "-c",
                       "from smurfs import Smurfs;import pickle;star : Smurfs = pickle.load(open('i_obj.smurfs', 'rb'));import os;os.remove('i_obj.smurfs');import matplotlib.pyplot as pl;pl.ion()"]
                subprocess.call(cmd)
                mprint("Done", info)
        else:
            if len(targets[0].split(".")) == 2 and os.path.basename(targets[0]).split(".")[1] in ['txt', 'dat']:
                s = MultiSmurfs(file_list=targets)
            else:
                s = MultiSmurfs(target_list=targets, flux_types=args.fluxType)

            s.run(snr=args.snr, window_size=args.windowSize, f_min=f_min, f_max=f_max,
                  skip_similar=args.skipSimilarFrequencies, similar_chancel=not args.skipCutoff
                  , extend_frequencies=args.extendFrequencies, improve_fit=args.disableImproveFrequencies
                  , mode=args.fitMethod)
            s.save(args.savePath, args.storeObject)


if __name__ == "__main__":
    main()



import lightkurve as lk
import astropy.units as u
from typing import Union
from matplotlib.axes import Axes

class LightCurve (lk.TessLightCurve):
    """
    Custom LightCurve object. Reimplements the plot routine, as we only use mag within SMURFS

    :param lc: Lightcurve object
    """
    def __init__(self, lc : Union[lk.TessLightCurve,lk.LightCurve]):
        if isinstance(lc,lk.TessLightCurve):
            super().__init__(time=lc.time,flux=lc.flux,flux_err=lc.flux_err,flux_unit=lc.flux_unit,time_format=lc.time_format,
                             centroid_col=lc.centroid_col,centroid_row=lc.centroid_row,quality=lc.quality,
                             quality_bitmask=lc.quality_bitmask,cadenceno=lc.cadenceno,sector=lc.sector,
                             camera=lc.camera, ccd=lc.ccd,targetid=lc.targetid,ra=lc.ra,dec=lc.dec,label=lc.label,
                             meta=lc.label)
        else:
            super().__init__(time=lc.time,flux=lc.flux,flux_err=lc.flux_err,time_format=lc.time_format,
                             time_scale=lc.time_scale,targetid=lc.targetid,label=lc.label,meta=lc.meta)

    def plot(self,**kwargs):
        ax: Axes = super().plot(color='k', ylabel="Flux [mag]", normalize=False, **kwargs)
        ax.set_ylim(ax.get_ylim()[::-1])

        return ax

    def scatter(self,**kwargs):
        ax: Axes = super().scatter(color='k', ylabel="Flux [mag]", normalize=False, **kwargs)
        ax.set_ylim(ax.get_ylim()[::-1])

        return ax


from lightkurve import search_lightcurvefile, TessLightCurveFile, LightCurveCollection
import lightkurve as lk
from typing import List, Tuple, Union
from astroquery.simbad import Simbad
from astropy.coordinates import SkyCoord
import astropy.units as u
import re
import eleanor
from astroquery.mast import Tesscut
from astroquery.mast import Catalogs
import numpy as np
import astropy.units as u
from smurfs.support.mprint import *
from contextlib import redirect_stdout
import io
from matplotlib.figure import Figure
from matplotlib.axes import Axes
from matplotlib.gridspec import GridSpec
import matplotlib.pyplot as pl
from astroquery.mast import Catalogs
from astroquery.mast import Observations
import astropy.units as u
from matplotlib import rcParams
import warnings
from smurfs.signal.lightcurve import LightCurve
from eleanor.visualize import Visualize

params = {
    'axes.labelsize': 16,
    #   'text.fontsize': 8,
    'legend.fontsize': 18,
    'xtick.labelsize': 18,
    'ytick.labelsize': 18,
    'text.usetex': False,
    'figure.figsize': [4.5, 4.5]
}
rcParams.update(params)

def pixel_by_pixel(data : eleanor.TargetData, whole_cutout : bool=False):
    """
    Plots light curves pixel by pixel, either on the full cutout or only the aperture

    :param data: data object
    :param whole_cutout: Defines if the whole cutout or only the aperture is used
    """
    aperture_mask = data.aperture > 0
    indx_mask = [[i for i,masked in enumerate(mask_row) if (masked or whole_cutout)] for mask_row in aperture_mask]
    if whole_cutout:
        xrange  = np.arange(0,aperture_mask.shape[0])
        yrange = np.arange(0,aperture_mask.shape[1])
    else:
        xrange = np.unique(np.where(aperture_mask)[0])
        yrange = np.unique(np.where(aperture_mask)[1])

    fig,ax_list = pl.subplots(ncols = len(xrange),nrows=len(yrange),figsize=(8.27,11.69),dpi=100)
    fig : Figure
    ax_list : List[Axes]
    q = data.quality == 0
    [[ax.axis('off') for ax in ax_list_col] for ax_list_col in ax_list]
    [[ax.tick_params(axis='both',which='both',bottom=False,top=False,left=False,right=False, labelbottom=False,labelleft=False) for ax in ax_list_col] for ax_list_col in ax_list]
    for row,col_list in enumerate(indx_mask):
        for col in col_list:
            flux = data.corrected_flux(flux = data.tpf[:,row,col])
            time = data.time
            y = flux[q]/np.nanmedian(flux[q])
            x = time[q]
            ax : Axes = ax_list[row-min(xrange)][col-min(yrange)]
            c = 'red' if aperture_mask[row][col] and whole_cutout else 'k'
            ax.plot(x,y,'o',c=c,markersize=1)
            ax.axis('on')
    fig.suptitle("Pixel by pixel light curve " + ("(Aperture only)" if not whole_cutout else "(full cutout)"))
    return fig

def aperture_contour(ax: Axes, obj: eleanor.TargetData):
    """
    Plots a countour plot of the TPF with the aperture by Eleanor.

    :param ax: Axes object where this is plotted on
    :param obj: TargetData object containing TPF
    :return: used axes object
    """
    aperture = obj.aperture

    ax.imshow(obj.tpf[0],cmap='binary')

    f = lambda x, y: aperture[int(y), int(x)]
    g = np.vectorize(f)

    x = np.linspace(0, aperture.shape[1], aperture.shape[1] * 100)
    y = np.linspace(0, aperture.shape[0], aperture.shape[0] * 100)
    X, Y = np.meshgrid(x[:-1], y[:-1])
    Z = g(X[:-1], Y[:-1])

    ax.contour(Z[:-1], [0.5], colors='r', linewidths=[4],
               extent=[0 - 0.5, x[:-1].max() - 0.5, 0 - 0.5, y[:-1].max() - 0.5])


def create_validation_page(data_list: List[eleanor.TargetData], q_list: List[np.ndarray],
                           title: str,do_psf = False, do_pca = False,flux_type : str = "PDCSAP") -> List[Figure]:

    fig_main: Figure = pl.figure(figsize=(11.69,8.27),dpi=100)
    ax : Axes= fig_main.add_subplot()

    for data, q in zip(data_list, q_list):
        if flux_type == 'PDCSAP':
            ax.plot(data.time[q], data.pca_flux[q] - np.median(data.pca_flux[q]), '.', markersize=2,
                    label=f"Sector {data.source_info.sector}")
        elif flux_type == 'PSF':
            ax.plot(data.time[q], data.psf_flux[q] - np.median(data.psf_flux[q]), '.', markersize=2,
                    label=f"Sector {data.source_info.sector}")
        else:
            ax.plot(data.time[q], data.corr_flux[q] - np.median(data.corr_flux[q]), '.', markersize=2,
                    label=f"Sector {data.source_info.sector}")
    ax.legend()
    ax.set_xlabel("Time")
    ax.set_ylabel("Flux")
    fig_main.suptitle(f"{title} - extracted light curve, flux type: {flux_type}")
    pl.tight_layout()

    reduction_figs = []

    for i, (data, q) in enumerate(zip(data_list, q_list)):
        fig : Figure= pl.figure(figsize=(8.27,11.69),dpi=100)
        gs = GridSpec(3,3, width_ratios=[1, 1, 1], height_ratios=[3,3,2])

        ax: Axes = fig.add_subplot(gs[0, 0:])

        ax.plot(data.time[q], data.raw_flux[q]/np.nanmedian(data.raw_flux[q])+0.2, 'ok',label='Raw flux',markersize=1)
        ax.plot(data.time[q], data.corr_flux[q]/np.nanmedian(data.corr_flux[q]) + 0.1, 'or',label='Corrected flux',markersize=1)
        if do_pca:
            ax.plot(data.time[q], data.pca_flux[q]/np.nanmedian(data.pca_flux[q]), 'og',label='PCA flux',markersize=1) #aperture × TPF + background subtraction + cotrending basis vectors
        if do_psf:
            ax.plot(data.time[q], data.psf_flux[q]/np.nanmedian(data.psf_flux[q]) - 0.1, 'ob',label='PSF modelled flux',markersize=1)
        ax.legend()

        ax: Axes = fig.add_subplot(gs[1, 0:])
        ax.plot(data.time, data.flux_bkg, 'k', label='1D postcard', linewidth=2)
        ax.plot(data.time, data.tpf_flux_bkg, 'r--', label='1D TPF', linewidth=1)
        ax.legend()
        ax.set_title("Background flux")

        ax: Axes = fig.add_subplot(gs[2, 0])
        aperture_contour(ax, data)
        ax.set_title("TPF with aperture")
        ax.set_xticks([])
        ax.set_yticks([])

        ax: Axes = fig.add_subplot(gs[2, 1])
        tpf_data = data.tpf[0]
        tpf_data[data.aperture != 1] = 0
        ax.imshow(tpf_data,cmap='binary')
        ax.set_title("Aperture content")
        ax.set_yticks([])
        ax.set_xticks([])

        ax: Axes = fig.add_subplot(gs[2, 2])
        ax.imshow(data.bkg_tpf[0],cmap='binary')
        ax.set_title("Interpolated background")
        ax.set_yticks([])
        ax.set_xticks([])

        fig.suptitle(f"{title} - Sector {data.source_info.sector}")
        pl.tight_layout()

        reduction_figs.append(fig)
        reduction_figs.append(pixel_by_pixel(data))
        reduction_figs.append(pixel_by_pixel(data,True))

    return  [fig_main] + reduction_figs

from uncertainties import unumpy as unp, ufloat


def mag(lc: LightCurve) -> LightCurve:
    """
    Converts and normalizes a LighCurve object to magnitudes.

    :param lc: lightcurve object
    :return: reduced light curve object
    """

    lc = lc.remove_nans()

    flux = lc.flux + (np.abs(2 * np.amin(lc.flux)) if np.amin(lc.flux) < 0 else 100)
    flux = unp.uarray(flux, lc.flux_err)
    flux = -2.5 * unp.log10(flux)
    flux = flux[~unp.isnan(flux)]
    flux -= np.median(flux)

    lc.flux = unp.nominal_values(flux) * u.mag
    lc.flux_err = unp.std_devs(flux) * u.mag
    return lc


def cut_ffi(tic_id:int,clip :float = 4,iter : int = 1,do_pca : bool = False, do_psf :bool = False,flux_type = 'PDCSAP') -> Tuple[LightCurve, List[Figure]]:
    """
    Extracts light curves from FFIs using TESScut and Eleanor. This function automatically combines all available
    sectors for a given target.

    :param tic_id: TIC ID of the target
    :param clip: Sigma clip range of the target.
    :param iter: Iterations of the sigma clipping
    :param do_pca: Perform pca analysis with eleanor
    :param do_psf: Perform psf analysis with eleanor
    :param flux_type: Flux type that is returned. Choose between 'PDCSAP','SAP','PSF'
    :return: Lightcurve
    """
    flux_types = ['SAP','PDCSAP', 'PSF']
    if flux_type not in flux_types:
        raise ValueError(mprint(f"Flux type {flux_type} not recognized. Possible values are: {flux_types}",error))
    f = io.StringIO()
    with redirect_stdout(f):
        stars = eleanor.multi_sectors(tic=tic_id, sectors='all',tc=True)
    mprint(f.getvalue().strip(), log)

    lc_list = []
    data_list = []
    q_list = []

    pca_flag = do_pca or flux_type == 'PDCSAP'
    psf_flag = do_psf or flux_type == 'PSF'

    for star in stars:

        f = io.StringIO()
        with warnings.catch_warnings():
            warnings.simplefilter('ignore')
            data = eleanor.TargetData(star, height=15, width=15, bkg_size=31, do_pca=pca_flag, do_psf=psf_flag)
        mprint(f.getvalue().strip(), log)
        q = data.quality == 0
        if flux_type == 'SAP':
            lc_list.append(LightCurve(lk.TessLightCurve(time=data.time[q], flux=data.corr_flux[q], targetid=tic_id)))
        elif flux_type == 'PDCSAP':
            lc_list.append(LightCurve(lk.TessLightCurve(time=data.time[q], flux=data.pca_flux[q], targetid=tic_id)))
        else:
            lc_list.append(LightCurve(lk.TessLightCurve(time=data.time[q], flux=data.psf_flux[q], targetid=tic_id)))

        data_list.append(data)
        q_list.append(q)

    fig = create_validation_page(data_list, q_list, f'TIC {tic_id}',do_pca=pca_flag,do_psf=psf_flag,flux_type=flux_type)

    lc : LightCurve = combine_light_curves(lc_list,clip,iter)
    mprint(f"Extracted light curve for TIC {tic_id}!", info)
    return lc, fig

def combine_light_curves(target_list: List[Union[lk.TessLightCurve, lk.KeplerLightCurve]],sigma_clip :float = 4,iters : int = 1) -> LightCurve:
    kepler_collection = LightCurveCollection([i for i in target_list if isinstance(i, lk.KeplerLightCurve)])
    tess_collection = LightCurveCollection([i for i in target_list if isinstance(i, lk.TessLightCurve)])

    if len(kepler_collection) != 0 and len(tess_collection) != 0:
        kepler_lc: lk.KeplerLightCurve = kepler_collection.stitch(corrector_func=mag)
        tess_lc: lk.TessLightCurve = tess_collection.stitch(corrector_func=mag)

        return LightCurve(kepler_lc.remove_nans().remove_outliers(sigma_clip,maxiters=iters).append(tess_lc.remove_nans().remove_outliers(4,maxiters=1)))
    elif len(kepler_collection) != 0:
        return LightCurve(kepler_collection.stitch(corrector_func=mag).remove_nans().remove_outliers(sigma_clip,maxiters=iters))
    elif len(tess_collection) != 0:
        return LightCurve(tess_collection.stitch(corrector_func=mag).remove_nans().remove_outliers(sigma_clip,maxiters=iters))
    else:
        raise ValueError(ctext("No light curves available for target!",error))


def download_lc(target_name: str, flux_type='PDCSAP', mission: str = 'TESS',sigma_clip=4,iters=1,do_pca : bool = False,do_psf :bool= False) -> Tuple[
    LightCurve, Union[List[Figure],None]]:
    """
    Downloads a light curve using the TESS mission. If the star has been observed in the SC mode, it
    will download the original light curve from MAST. You can also choose the flux type you want to use.

    If it wasn't observed in SC mode, it will try to extract a light curve from the FFIs if the target has
    been observed by TESS.

    You can also download light curves of stars that are observed by the K2 or Kepler mission, by setting
    the mission parameter.

    :param target_name: Name of the target. You can either provide the TIC ID (TIC ...), Kepler ID (KIC ...), K2 ID(EPIC ...) or a name that is resolvable by Simbad.
    :param flux_type: Type of flux in the SC mode. Can be either PDCSAP or SAP or PSF for long cadence data
    :param mission: Mission from which the light curves are extracted. By default TESS only is used. You can consider all missions by passing 'all' (TESS, Kepler, K2)
    :param sigma_clip: Sigma clip parameter. Defines the number of standard deviations that are clipped.
    :param iters: Iterations for the sigma clipping
    :return: lightkurve.LightCurve object and validation page if extracted from FFI
    """
    chosen_mission = [mission] if mission != 'all' else ('Kepler', 'K2', 'TESS')
    mprint(f"Searching processed light curves for {target_name} on mission(s) {','.join(chosen_mission)} ... ", log)

    if chosen_mission == ['TESS']:
        if target_name.startswith('TIC'):
            tic_id = re.findall(r'\d+', target_name)
            if len(tic_id) == 0:
                raise ValueError(ctext("A Tic ID needs to consist of TIC and a number!", error))
            tic_id = int(tic_id[0])
        else:
            mprint(f"Resolving {target_name} to TIC using MAST ...",log)
            try:
                tic_id = Catalogs.query_object(target_name,catalog='TIC',radius=0.003)[0]['ID']
            except KeyError:
                raise ValueError(ctext(f"No TESS observations available for {target_name}", error))

            mprint(f"TIC ID for {target_name}: TIC {tic_id}",log)

        o = Observations.query_criteria(objectname=target_name, radius=str(0 * u.deg), project='TESS',
                                        obs_collection='TESS').to_pandas()

        if len(o) > 0 and len(o[o.target_name != 'TESS FFI']) > 0:
            mprint(f"Short cadence observations available for {target_name}. Downloading ...",info)
            res = search_lightcurvefile(target_name, mission=chosen_mission)
        else: #Only FFI available
            mprint(f"No short cadence data available for {target_name}, extracting from FFI ...",info)
            lc, fig = cut_ffi(tic_id,sigma_clip,iters,do_pca,do_psf,flux_type)
            mprint(f"Total observation length: {'%.2f' % (lc.time[-1] - lc.time[0])} days.", log)
            return lc, fig
    else:
        res = search_lightcurvefile(target_name, mission=chosen_mission)

    if len(res) != 0:
        fig = None
        mprint(f"Found processed light curve for {target_name}!", info)
        res = res.download_all()
        types = []

        for d in res.data:
            type = 'TESS' if isinstance(d, lk.TessLightCurveFile) else 'Kepler'
            if type not in types:
                types.append(type)

        mprint(f"Using {','.join(types)} observations! Combining sectors ...", log)

        if flux_type == 'PSF':
            mprint(f"PSF not available for short cadence data. Reverting to PDCSAP",warn)
            flux_type = 'PDCSAP'

        if flux_type == 'PDCSAP':
            lc_set: List[Union[lk.TessLightCurve, lk.KeplerLightCurve]] = [i for i in res.PDCSAP_FLUX.data]
        elif flux_type == 'SAP':
            lc_set: List[Union[lk.TessLightCurve, lk.KeplerLightCurve]] = [i for i in res.SAP_FLUX.data]
        else:
            raise ValueError(ctext("Flux type needs to be either PDCSAP or SAP", error))
        lc = combine_light_curves(lc_set,sigma_clip=sigma_clip,iters=iters)
    else:
        raise ValueError(ctext(f"No light curve available for {target_name} on mission(s) {chosen_mission}",error))

    mprint(f"Total observation length: {'%.2f' % (lc.time[-1] - lc.time[0])} days.", log)
    return lc, fig



import os
import numpy as np
from lightkurve import LightCurve
from smurfs.preprocess.tess import mag
from smurfs.support.mprint import *
from pandas import read_csv


def load_file(file: str, clip: float = 4, it: int = 1, apply_file_correction: bool = False) -> LightCurve:
    """
    Loads and normalizes target content
    :param file: Name of target including path
    :return: LightCurve object
    """
    if not os.path.exists(file):
        raise IOError(ctext(f"File {file} doesn't exist!", error))

    mprint(f"Reading data from {file} ...", log)
    try:
        data = np.loadtxt(file)
    except ValueError:
        data = read_csv(file)
        data = np.array((data.time,data.flux))
    if data.shape[0] > data.shape[1]:
        data = data.T

    if data.shape[0] == 2:
        lc = LightCurve(time=data[0], flux=data[1])
    else:
        lc = LightCurve(time=data[0], flux=data[1], flux_err=data[2])

    lc = lc.remove_nans()
    if apply_file_correction:
        lc.flux = lc.flux + float(np.amin(lc.flux)) + 10
        lc = lc.remove_outliers(clip, maxiters=it)
        lc = mag(lc)
        lc = lc.remove_nans()
    else:
        if np.amax(np.abs(lc.flux)) > 10:
            mprint(
                f"It seems as if your flux isn't in magnitudes. Be aware, that SMURFS expects the flux in magnitudes. "
                f"Continuing ...",
                warn)
        if np.abs(np.median(lc.flux)) > 1:
            mprint(
                f"The median of your flux is {'%.2f' % np.median(lc.flux)}. To do a proper analysis, the median should "
                f"be close to 0. Be aware, that this might cause issues. Continuing...",
                warn)
    mprint(f"Total observation length: {'%.2f' % (lc.time[-1] - lc.time[0])} days.", log)
    mprint("Extracted data from target!", info)
    return lc
